%% COMPRESSOR CHARACTERISTIC APPROXIMATION
%%
%% MATLAB file which generates a bivariate spline.  This spline is stored
%% in the file "psic_data" in the current directory and can be accessed by the
%% command "pp = load('psic_data')".
%% From this spline the value of the compressor characteristic can be returned
%% for all speeds and massflows.
%%
%% AUTHOR: Bjrn Ove Barstad



% STEP 1 - collect points from the compressor map
x_corrected = {};
y = {};

%points for the speedline 20000 rpm
x_corrected(1) = {[10 20 30 40]};
y(1) = {[1.175 1.185 1.18 1.13]};

%points for the speedline 25000 rpm
x_corrected(2) = {[10 20 30 40 50 60]};
y(2) = {[1.275 1.30 1.305 1.285 1.245 1.15]};

%points for the speedline 30000 rpm
x_corrected(3) = {[20 30 40 50 60]};
y(3) = {[1.435 1.45 1.445 1.42 1.365]};

%points for the speedline 35000 rpm
x_corrected(4) = {[20 30 40 50 60 70]};
y(4) = {[1.58 1.617 1.625 1.617 1.57 1.48]};

%points for the speedline 40000 rpm
x_corrected(5) = {[30 40 50 60 70]};
y(5) = {[1.81 1.83 1.825 1.8 1.733]};

%points for the speedline 45000 rpm
x_corrected(6) = {[40 50 60 70 80]};
y(6) = {[2.09 2.095 2.09 2.06 1.93]};

%points for the speedline 50000 rpm
x_corrected(7) = {[50 60 70 80]};
y(7) = {[2.385 2.4 2.335 2.17]};

%plot
X_CORRECTED = [];
Y = [];
for i = 1 : length(x_corrected)
    for j = 1 : length(x_corrected{i})
        X_CORRECTED = [X_CORRECTED x_corrected{i}(j)];
        Y = [Y y{i}(j)];
    end
end
scrsz = get(0,'ScreenSize');
figure(1)
set(1,'Position',[(scrsz(3)/2 - 466/2) (scrsz(4)/2 - 411/2) 466 411]);
plot(X_CORRECTED, Y,'+'),grid,axis([-10 90 1.0 2.4])
xlabel('Corrected mass flow lb/min')
ylabel('Pressure ratio')



% STEP 2 - augment the points with zero flow - and negative flow points

%kappa = c_p / c_v, air
kappa = 1.4;
%sonic velocity
c_plenum = 343;
%ambient temperature 30 deg C
T_01 = 303.15;
%inducer diameter
D1 = 0.079;
%impeller perimeter diameter
D2 = 0.113;

%speedvector
N = [20000 25000 30000 35000 40000 45000 50000]; %RPM

%zero flow pressure ratio
PSI_czero = [];
for i = 1 : length(N)
    PSI_czero = [PSI_czero (1 + (pi^2* (N(i)/60)^2 *(D2^2 - D1^2))/(2*c_plenum*T_01))^(kappa/(kappa - 1))];
end

%negative flow pressure ratio
PSI_cneg = [1.19 1.30 1.45 1.55 1.70 1.90 2.12];

%augment
for i = 1 : length(x_corrected)
    x_corrected(i) = {[-10 0 x_corrected{i}]};
    y(i) = {[PSI_cneg(i) PSI_czero(i) y{i}]};
end

%generate new points to plot
X_CORRECTED = [];
Y = [];
for i = 1 : length(x_corrected)
    for j = 1 : length(x_corrected{i})
        X_CORRECTED = [X_CORRECTED x_corrected{i}(j)];
        Y = [Y y{i}(j)];
    end
end



% STEP 3 - cubic bivariate spline interpolation and extrapolation based on tensor product splines

%generate a cubic spline in ppform for each speedline
PP = [];
for i = 1 : length(x_corrected)
    PP = [PP spline(x_corrected{i}, y{i})];
end

%prepare grid {w, omega} = {massflow, speed}
w = -10 : 10 : 100;
omega = 20000 : 5000 : 50000;

%construct the z-matrix z(i,j) = f( w(i) , omega(j) ) ( = PSI_c )
%from the one-dim splines
z = [];
for i = 1 : length(omega)
    for j = 1 : length(w)
        z(j,i) = fnval(PP(i), w(j));
    end
end

%cubic bivariate spline in ppform generation
pp = csape({w, omega}, z,{'clamped', 'clamped'});

%gather lines to draw
w = -10 : 1 : 80;
clear PP
PP = [];
for i = 1 : length(N)
    PP = [PP fnval(pp, {w, N(i)})];
end
PP(:,8) = fnval(pp, {w, 29500});

%plot points and approximated speedlines
scrsz = get(0,'ScreenSize');
figure(2)
set(2,'Position',[(scrsz(3)/2 - 466/2) (scrsz(4)/2 - 411/2) 466 411]);
plot(X_CORRECTED, Y,'+', w, PP(:,1), w, PP(:,2), w, PP(:,3), w, PP(:,4), w, PP(:,5), w, PP(:,6), w, PP(:,7)),grid,axis([-10 90 1.0 2.4])
xlabel('Corrected mass flow lb/min')
ylabel('Pressure ratio')



% STEP 4 - save the only data needed, pp, in a file
save('psic_data','-struct','pp');
clear all
