%% COMPRESSOR CHARACTERISTIC APPROXIMATION
%%
%% Compressor type: Vortech S-trim
%%
%% MATLAB file which generates an approximation matrix "c". This matrix is stored as
%% "c_matrix" and can be accessed by the command "c = dlmread('c_matrix')".
%% From this matrix the value of the compressor characteristic can be returned
%% for all speeds and massflows.
%%
%% AUTHOR: Bjrn Ove Barstad

% CALCULATION OF ZERO FLOW PRESSURE RATIO

%values needed for eq. 13.50
%in Gravdahl og Egeland 2002

%kappa = c_p / c_v, air
kappa = 1.4;
%sonic velocity
c_plenum = 343;
%ambient temperature 30 deg C
T_01 = 303.15;
%inducer diameter
D1 = 0.079;
%impeller perimeter diameter
D2 = 0.119;

%speedvector
N = [20000 25000 30000 35000 40000 45000 50000]; %RPM

%zero flow pressure ratio
%eq. 13.50 in Gravdahl & Egeland 2002
PSI_czero = [];
for i = 1 : length(N)
    PSI_czero = [PSI_czero (1 + (pi^2* (N(i)/60)^2 *(D2^2 - D1^2))/(2*c_plenum*T_01))^(kappa/(kappa - 1))];
end

% MANUAL INSERTION OF POINTS
% FOR DIFFERENT SPEEDLINES
x_corrected = {};
y = {};

%points for the speedline 20000 rpm
x_corrected(1) = {[-10 0 10 20 30 40]};
y(1) = {[1.19 PSI_czero(1) 1.175 1.185 1.18 1.13]};

%points for the speedline 25000 rpm
x_corrected(2) = {[-10 0 10 20 30 40 50]};
y(2) = {[1.37 PSI_czero(2) 1.275 1.30 1.305 1.285 1.245]};

%points for the speedline 30000 rpm
x_corrected(3) = {[-10 0 20 30 40 50 60]};
y(3) = {[1.45 PSI_czero(3) 1.435 1.45 1.445 1.42 1.365]};

%points for the speedline 35000 rpm
x_corrected(4) = {[-10 0 20 30 40 50 60 70]};
y(4) = {[1.55 PSI_czero(4) 1.58 1.617 1.625 1.617 1.57 1.48]};

%points for the speedline 40000 rpm
x_corrected(5) = {[-10 0 30 40 50 60 70]};
y(5) = {[1.80 PSI_czero(5) 1.81 1.83 1.825 1.8 1.733]};

%points for the speedline 45000 rpm
x_corrected(6) = {[-10 0 40 50 60 70 80]};
y(6) = {[2.10 PSI_czero(6) 2.09 2.095 2.09 2.06 1.93]};

%points for the speedline 50000 rpm
x_corrected(7) = {[-10 0 50 60 70 80]};
y(7) = {[2.30 PSI_czero(7) 2.385 2.4 2.335 2.17]};

X_CORRECTED = [];
Y = [];
for i = 1 : length(x_corrected)
    for j = 1 : length(x_corrected{i})
        X_CORRECTED = [X_CORRECTED x_corrected{i}(j)];
        Y = [Y y{i}(j)];
    end
end

scrsz = get(0,'ScreenSize');
figure(1);
set(1,'Position',[(scrsz(3)/2 - 466/2) (scrsz(4)/2 - 411/2) 466 411]);
plot(X_CORRECTED, Y,'+'),grid,axis([-10 90 1.0 2.4])
xlabel('Corrected mass flow lb/min')
ylabel('Pressure ratio')

% THIRD ORDER POLYNOMIAL APPROXIMATION
% OF THE SPEEDLINES
%the speedlines will then be continuous
%in mass flow
POLY = [];
for i = 1 : length(x_corrected)
    POLY = [POLY; polyfit(x_corrected{i},y{i},3)];
end


% POLYNOMIAL APPROXIMATION OF THE COEFFICIENTS
%the approximation will then be continuous
%in rotational speed
c = [];
for i = 1 : 4
    c = [c; polyfit(N,[POLY(1,i) POLY(2,i) POLY(3,i) POLY(4,i) POLY(5,i) POLY(6,i) POLY(7,i)],3)];
end


dlmwrite('c_matrix', c);
clear all
